﻿using System;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls;
using System.Linq;

namespace TreeViewMVVMCheckBoxSample.ViewModels
{
    public class CategoryViewModel : ViewModelBase
    {
        private string _name;
        private bool? _isChecked;
        private bool reentrancyCheck = false;
        private CategoryViewModel parentItem;

        private ObservableCollection<CategoryViewModel> _subCategories = null;

        public string Name
        {
            get
            {
                return this._name;
            }
            set
            {
                this._name = value;
            }
        }
        public bool? IsChecked
        {
            get
            {
                return this._isChecked;
            }
            set
            {
                if (this._isChecked != value)
                {
                    if (reentrancyCheck)
                        return;
                    this.reentrancyCheck = true;
                    this._isChecked = value;
                    this.UpdateCheckState();
                    OnPropertyChanged("IsChecked");
                    this.reentrancyCheck = false;
                }
            }
        }

        public ObservableCollection<CategoryViewModel> SubCategories
        {
            get
            {
                if (this._subCategories == null)
                {
                    this._subCategories = new ObservableCollection<CategoryViewModel>();
                }
                return this._subCategories;
            }
        }

        public CategoryViewModel(CategoryViewModel parent)
        {
            this.parentItem = parent;
        }

        private void UpdateCheckState()
        {
            // update all children:
            if (this.SubCategories.Count != 0)
            {
                this.UpdateChildrenCheckState();
            }
            //update parent item
            if (this.parentItem != null)
            {
                bool? parentIsChecked = this.parentItem.DetermineCheckState();
                    this.parentItem.IsChecked = parentIsChecked;

            }
        }

        private void UpdateChildrenCheckState()
        {
            foreach (var item in this.SubCategories)
            {
                if (this.IsChecked != null)
                {
                    item.IsChecked = this.IsChecked;
                }
            }
        }

        private bool? DetermineCheckState()
        {
            bool allChildrenChecked = this.SubCategories.Count(x => x.IsChecked == true) == this.SubCategories.Count;
            if (allChildrenChecked)
            {
                return true;
            }

            bool allChildrenUnchecked = this.SubCategories.Count(x => x.IsChecked == false) == this.SubCategories.Count;
            if (allChildrenUnchecked)
            {
                return false;
            }

            return null;
        }
    }
}
